# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Haik Mohammed <haikm@inbox.ru>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import locale, gettext

from gi.repository import Gtk, Gdk, GLib # pylint: disable=E0611
import logging
logger = logging.getLogger('openselfbondage')

from openselfbondage_lib import Window
from openselfbondage.AboutOpenselfbondageDialog import AboutOpenselfbondageDialog
from openselfbondage.PreferencesOpenselfbondageDialog import PreferencesOpenselfbondageDialog

import sys
import commands
import threading,time,datetime

locale.setlocale(locale.LC_ALL, '')
LOCALE_DIR = 'moo/'
LOCALE_DOMAIN = 'openselfbondage'
#locale.bindtextdomain(LOCALE_DOMAIN, LOCALE_DIR)
gettext.bindtextdomain(LOCALE_DOMAIN, LOCALE_DIR)   # (1)
gettext.textdomain(LOCALE_DOMAIN)                   # (2)
#gettext.install(LOCALE_DOMAIN)                      # (3)
_ = gettext.gettext

# See openselfbondage_lib.Window.py for more details about how this class works
class OpenselfbondageWindow(Window):
    __gtype_name__ = "OpenselfbondageWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(OpenselfbondageWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutOpenselfbondageDialog
        self.PreferencesDialog = PreferencesOpenselfbondageDialog

        # Code for other initialization actions should be added here.
        self.playbutton = self.builder.get_object("bPlay")
        self.toolbar1 = self.builder.get_object("toolbar1")
        self.cbdrivelist = self.builder.get_object("cbDriveList")
        self.spinHr = self.builder.get_object("spinHr")
        self.spinMn = self.builder.get_object("spinMn")
        self.spinSe = self.builder.get_object("spinSe")
        self.bTest = self.builder.get_object("bTest")
        self.sbar = self.builder.get_object("statusbar1")

        context = self.toolbar1.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)

        self.infobar = self.sbar.get_context_id('1')
        self.sbar.push(self.infobar,_('Program launched successfully!'))

        #enum CD trays
        output = commands.getoutput("dmesg | egrep -i --color 'cdrom|dvd|cd/rw|writer'")
        L = self.parse_output(output)

        if L == "0":
            self.messageDialog(_("No CD Drives were detected, this software can't continue!"))
            sys.exit(1)

        for string in L:
            self.cbdrivelist.append_text(string)


    def on_bPlay_clicked(self, widget):
        if not self.cbdrivelist.get_active_text():
            self.messageDialog(_("Please select an entry to eject"))
            return
        self.playbutton.set_sensitive(False)
        self.release()

    def on_bTest_clicked(self, widget):
        self.ejectDrive()
        md = Gtk.MessageDialog(self,Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.INFO,Gtk.ButtonsType.YES_NO, _("Did the tray open?"))
        res = md.run()
        if res == -9:
            self.messageDialog(_("You shouldn't use this software then!, check your CD-Drive connections/physical stability and try again!"))
        md.destroy()

    def parse_output(self, string):
        drive_mount_points = list()
        try:
            for x in range(0, 5):
                if 'sr'+str(x) + ':' in string:
                    drive_mount_points.append("/dev/sr" + str(x))

        except ValueError:
            pass

        if len(drive_mount_points) == 0:
            return '0'        

        return drive_mount_points

    def release(self):
        win = TimerWindow()
        win.show_all()
        win.maximize()

        t = WorkingSlave(self.spinHr.get_value(),self.spinMn.get_value(),self.spinSe.get_value(),win,self)
        t.start()

    def ejectDrive(self):
        cmd = "eject " + self.cbdrivelist.get_active_text()
        output = commands.getoutput(cmd.strip())

    def messageDialog(self, msg):
        md = Gtk.MessageDialog(self,Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.INFO,Gtk.ButtonsType.CLOSE, msg)
        md.run()
        md.destroy()

class WorkingSlave(threading.Thread):
    def __init__(self,h,m,s,c,cc):
        self.__h = h
        self.__m = m
        self.__s = s
        self.__c = c
        self.__cc = cc
        threading.Thread.__init__(self)

    def run(self):
        while True:
            time.sleep(1)
            self.__s -= 1

            if self.__s < 0:
                self.__s = 59
                self.__m -= 1

            if self.__m < 0:
                self.__m = 59
                self.__h -= 1

            if self.__h <=0 and self.__m <= 0 and self.__s <= 0:
                self.__c.hide()

                Gdk.threads_enter()
                self.__cc.ejectDrive()
                self.__cc.messageDialog(_("You are free!"))
                self.__cc.sbar.push(self.__cc.infobar,_("Session Ended"))
                self.__cc.playbutton.set_sensitive(True)
                Gdk.threads_leave()
                break

            #Update GUI
            #Gdk.threads_enter() were not needed here...
            self.__c.lTime.set_markup('<span size="144000">' + self.format(self.__h,self.__m,self.__s) + '</span>')
            #Gdk.threads_leave()

    def format(self,h,m,s):
        s = datetime.datetime.strptime(str(int(h)) + ":" + str(int(m)) + ":" + str(int(s)), "%H:%M:%S")
        return s.strftime("%H:%M:%S")

class TimerWindow(Gtk.Window):
   def __init__(self):
        Gtk.Window.__init__(self, title="Active SB Session")
        box = Gtk.Box(spacing=6)
        self.add(box)
        self.lTime = Gtk.Label()
        self.lTime.set_use_markup(True)
        self.lTime.set_markup('<span size="144000">00:00:00</span>')
        box.pack_start(self.lTime, True, True, 0)

